#include <windows.h>
#include <string.h>
#include "helputil.h"   
#include "bitmap.h"

HANDLE ReadInternalDibBitmapInfo( HF fh );
DWORD PASCAL lInternalread( HF fh, VOID FAR *pv, DWORD ul );

HANDLE OpenInternalDIB (LPSTR szFile)
{
    HF		        	fh;
    HFS					hFs;
    BITMAPINFOHEADER    bi;
    LPBITMAPINFOHEADER  lpbi;
    DWORD               dwLen = 0;
    DWORD               dwBits;
    HANDLE              hdib;
    HANDLE              h;
	static HelpInstance	HI;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);

    _fstrupr( szFile );

    /* Open the file and read the DIB information */   

    hFs = OpenHelpFileSystem();
    if ( ! hFs )
    	return NULL;
    fh = OpenHelpFile (hFs, szFile );
    if (!fh)
    	return NULL;
    hdib = ReadInternalDibBitmapInfo(fh);
    if (!hdib)
    	{
    	MessageBox( GetActiveWindow(), "O ficheiro no deve ser um DIB",
    		"Erro", 0 );
    	return NULL;
    	}
    DibInfo(hdib,&bi);

    /* Calculate the memory needed to hold the DIB */
    dwBits = bi.biSizeImage;
    dwLen  = bi.biSize + (DWORD)PaletteSize (&bi) + dwBits;

    /* Try to increase the size of the bitmap info. buffer to hold the DIB */
    h = GlobalReAlloc(hdib, dwLen, GHND);
    if (!h){
        GlobalFree(hdib);
        hdib = NULL;
    }
    else
        hdib = h;

    /* Read in the bits */
    if (hdib)
    	{

        lpbi = (VOID FAR *)GlobalLock(hdib);
        lInternalread(fh, (LPSTR)lpbi + (WORD)lpbi->biSize + PaletteSize(lpbi), dwBits);
        GlobalUnlock(hdib);
    }
    ThisHelp->RcCloseHf(fh);
    return hdib;
}

HANDLE ReadInternalDibBitmapInfo (HF fh)
{
	DWORD				off;
	HANDLE				hbi = NULL;
	int					size;
	int					i;
	WORD				nNumColors;
	RGBQUAD FAR			*pRgb;
	BITMAPINFOHEADER	bi;
	BITMAPCOREHEADER	bc;
	LPBITMAPINFOHEADER	lpbi;
	BITMAPFILEHEADER	bf;
	DWORD				dwWidth = 0;
	DWORD				dwHeight = 0;
	WORD				wPlanes, wBitCount;
	static HelpInstance	HI;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);

    if (fh == -1)
        return NULL;

    /* Reset file pointer and read file header */
    off = ThisHelp->LSeekHf(fh, 0L, wFSSeekCur);
    if (sizeof (bf) != ThisHelp->LcbReadHf(fh, (LPSTR)&bf, sizeof (bf)))
        return FALSE;

    /* Do we have a RC HEADER? */
    if (!ISDIB (bf.bfType))
    {
        bf.bfOffBits = 0L;
        ThisHelp->LSeekHf(fh, off, wFSSeekSet);
    }
    if (sizeof (bi) != ThisHelp->LcbReadHf(fh, (LPSTR)&bi, sizeof(bi)))
        return FALSE;

    nNumColors = DibNumColors (&bi);

    /* Check the nature (BITMAPINFO or BITMAPCORE) of the info. block
     * and extract the field information accordingly. If a BITMAPCOREHEADER,
     * transfer it's field information to a BITMAPINFOHEADER-style block
     */
    switch (size = (int)bi.biSize){
        case sizeof (BITMAPINFOHEADER):
            break;

        case sizeof (BITMAPCOREHEADER):

            bc = *(BITMAPCOREHEADER*)&bi;

            dwWidth   = (DWORD)bc.bcWidth;
            dwHeight  = (DWORD)bc.bcHeight;
            wPlanes   = bc.bcPlanes;
            wBitCount = bc.bcBitCount;

            bi.biSize               = sizeof(BITMAPINFOHEADER);
            bi.biWidth              = dwWidth;
            bi.biHeight             = dwHeight;
            bi.biPlanes             = wPlanes;
            bi.biBitCount           = wBitCount;

            bi.biCompression        = BI_RGB;
            bi.biSizeImage          = 0;
            bi.biXPelsPerMeter      = 0;
            bi.biYPelsPerMeter      = 0;
            bi.biClrUsed            = nNumColors;
            bi.biClrImportant       = nNumColors;

            ThisHelp->LSeekHf (fh, (LONG)sizeof (BITMAPCOREHEADER) - sizeof (BITMAPINFOHEADER), wFSSeekCur);
            break;

        default:
            /* Not a DIB! */
            return NULL;
    }

    /*  Fill in some default values if they are zero */
    if (bi.biSizeImage == 0){
        bi.biSizeImage = WIDTHBYTES ((DWORD)bi.biWidth * bi.biBitCount)
                         * bi.biHeight;
    }
    if (bi.biClrUsed == 0)
        bi.biClrUsed = DibNumColors(&bi);

    /* Allocate for the BITMAPINFO structure and the color table. */
    hbi = GlobalAlloc (GHND, (LONG)bi.biSize + nNumColors * sizeof(RGBQUAD));
    if (!hbi)
        return NULL;
    lpbi = (VOID FAR *)GlobalLock (hbi);
    *lpbi = bi;

    /* Get a pointer to the color table */
    pRgb = (RGBQUAD FAR *)((LPSTR)lpbi + bi.biSize);
    if (nNumColors){
        if (size == sizeof(BITMAPCOREHEADER)){
            /* Convert a old color table (3 byte RGBTRIPLEs) to a new
             * color table (4 byte RGBQUADs)
             */
            ThisHelp->LcbReadHf (fh, (LPSTR)pRgb, nNumColors * sizeof(RGBTRIPLE));

            for (i = nNumColors - 1; i >= 0; i--){
                RGBQUAD rgb;

                rgb.rgbRed      = ((RGBTRIPLE FAR *)pRgb)[i].rgbtRed;
                rgb.rgbBlue     = ((RGBTRIPLE FAR *)pRgb)[i].rgbtBlue;
                rgb.rgbGreen    = ((RGBTRIPLE FAR *)pRgb)[i].rgbtGreen;
                rgb.rgbReserved = (BYTE)0;

                pRgb[i] = rgb;
            }
        }
        else
            ThisHelp->LcbReadHf(fh,(LPSTR)pRgb,nNumColors * sizeof(RGBQUAD));
    }

    if (bf.bfOffBits != 0L)
        ThisHelp->LSeekHf(fh,off + bf.bfOffBits,wFSSeekSet);

    GlobalUnlock(hbi);
    return hbi;
}

DWORD PASCAL lInternalread (fh, pv, ul)
HF            fh;
VOID far      *pv;
DWORD         ul;
{
	DWORD				ulT = ul;
	BYTE huge			*hp = pv;
	static HelpInstance	HI;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);

	while (ul > (DWORD)MAXREAD)
	{
		if (ThisHelp->LcbReadHf(fh, (LPSTR)hp, (WORD)MAXREAD) != MAXREAD)
			return 0;
		ul -= MAXREAD;
		hp += MAXREAD;
	}
	if ((WORD)ThisHelp->LcbReadHf(fh, (LPSTR)hp, (WORD)ul) != (WORD)ul)
		return 0;
	return ulT;
}
