/*
 *  printall.c - DLL to provide multi-topic printing.
 *  Copyright (c) Ron Burk 1993
 */

#include <stdlib.h>
#include <windows.h>

#define WM_HELP_INQUIRE    (1)  /* asks what notifications you want */
#define WM_HELP_FUNCPTRS  (10)  /* supplies pointers to functions   */
#define HN_FUNCPTRS   (0x0010)  /* bit to receive WM_HELP_FUNCPTRS  */
#define ID_CANCEL		0x1000

typedef LONG (CALLBACK *WINHELPPTR)(LPSTR,WORD,DWORD);
typedef struct  HelpInstance
    {
    struct HelpInstance   *Next;
    HTASK           TaskHandle;
    WINHELPPTR      WinHelp;
    }           HelpInstance;
char buffer[100];
typedef struct arr_topicos
{
	char topico[50];
	int  n;
} ARRT;

int Canceled=FALSE;

#define MAXTOPICOS	300
ARRT	arr[MAXTOPICOS];
int	nTopicos=0;

HelpInstance *HelpList;

HelpInstance    *NewHelpInstance(FARPROC FAR *FuncPtrs)
    {
    HelpInstance    *This;
    This        = (HelpInstance *)malloc(sizeof(HelpInstance));
    if(This != NULL)
        {
        This->Next          = NULL;
        This->TaskHandle    = GetCurrentTask();
        This->WinHelp       = (WINHELPPTR)FuncPtrs[16];
        if(HelpList)
            {
            This->Next  = HelpList;
            HelpList    = This;
            }
        else    /* else first on list */
            HelpList    = This;
        }
    return This;
    }

HelpInstance    *CurrentHelpInstance(void)
    {
    HTASK   ThisTask        = GetCurrentTask();
    HelpInstance    *Rover  = HelpList;

    while(Rover)    /* search linked list for matching task */
        if(Rover->TaskHandle == ThisTask)
            return Rover;
        else
            Rover   = Rover->Next;
    return Rover;
    }


HINSTANCE   ModuleHandle;

long FAR PASCAL _export MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	HWND hChild;

	switch (msg)
	{
	case WM_CREATE:
		hChild = CreateWindow( "button", "OK", WS_CHILD|BS_PUSHBUTTON|BS_DEFPUSHBUTTON|WS_VISIBLE,
			70, 50, 60, 25,
			hWnd, ID_CANCEL, ModuleHandle, NULL );
		return TRUE;
	case WM_COMMAND:
		if(wParam==ID_CANCEL)
			{
			Canceled=TRUE;
			DestroyWindow( hWnd );
         }      	
		return (DefWindowProc(hWnd, msg, wParam, lParam));

	case WM_DESTROY:
		 break;

	default:
	    return (DefWindowProc(hWnd, msg, wParam, lParam));
    }
    return (NULL);

}

InitInstance()
{
	 WNDCLASS wc;

	 wc.style = 0;
    wc.lpfnWndProc = MainWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
	 wc.hInstance = ModuleHandle;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
    wc.lpszMenuName =  NULL;
    wc.lpszClassName = "PrintAllClass";

    return (RegisterClass(&wc));
}

/* LibMain - store module handle, register clipboard format */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK LibMain(HINSTANCE ModuleHandle_,
    WORD DataSegment, WORD HeapSize, LPSTR CommandLine)
    {
    if(HeapSize > 0)
        UnlockData(0);
	 ModuleHandle    = ModuleHandle_;
    return InitInstance();
	 }

/* WEP - return success */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK WEP(int ExitCode)
    {
    return 1;
    }


/* HelpYield() - Synchronize winhelp.exe macro execution */
static  void    HelpYield(void)
    {
    MSG     Message;

    while(PeekMessage(&Message, NULL, 0, 0, PM_REMOVE))
        {
        if(Message.message == WM_QUIT)
            {
            PostQuitMessage(0);
            return;
            }
        else
            {
            TranslateMessage(&Message);
            DispatchMessage(&Message);
				}
		  }
    }

/* PrintThis() - prints current topic in given help file */
void CALLBACK _export PAZ(char FAR* Topico, int n)
{
	if(nTopicos>MAXTOPICOS)
   	{
		MessageBox(GetActiveWindow(), "The maximum number of available topics was reached", "Error", 0 );
		return;
      }
	lstrcpy( arr[nTopicos].topico, (LPCSTR) Topico );
	arr[nTopicos++].n = n;
}

void CALLBACK _export PrintAll(char FAR *HelpFile)
{
	HelpInstance    *ThisHelp = CurrentHelpInstance();
	int i,j;
   HWND hWnd;

	hWnd=CreateWindow( "PrintAllClass", "Stop Printing", WS_DLGFRAME|WS_VISIBLE, 200, 175,
		200, 130, GetActiveWindow(), NULL, ModuleHandle, NULL );
	ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
		 (DWORD)(void FAR *)"SaveMark(`PrintAllMark')");
	HelpYield();
	for(j=0;j<nTopicos;j++)
   	{
		wsprintf(buffer, "JI(qchPath, \"%s\")", arr[j].topico );
		ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
			 (DWORD)(void FAR *)buffer);
		HelpYield();
		for(i=0;i<arr[j].n;i++)
		{
			 if(Canceled)
          	break;
			 ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
						(DWORD)(void FAR *)"Print()");
			 HelpYield();
			 if(Canceled)
          	break;
			 ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
						(DWORD)(void FAR *)"Next()");
			 HelpYield();
			 if(Canceled)
          	break;
		}
	}
	Canceled=FALSE;
	DestroyWindow(hWnd);
	ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
		 (DWORD)(void FAR *)"GoToMark(`PrintAllMark')");
	HelpYield();
	ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
		 (DWORD)(void FAR *)"DeleteMark(`PrintAllMark')");
	HelpYield();
}


/* LDLLHandler - Special function that WinHelp calls implicitly */
#ifdef __BORLANDC__
	 #pragma argsused
#endif        
LONG CALLBACK _export LDLLHandler(WORD Message, LONG Param1, LONG Param2)
    {
    switch(Message)
        {
        case WM_HELP_INQUIRE   :
            return HN_FUNCPTRS;
        case WM_HELP_FUNCPTRS  :
            NewHelpInstance((FARPROC FAR *)Param1);
            return TRUE;
        }
    return FALSE;
    }

