/*
 *  printall.c - WinHelp DLL to print all topics from a help file.
 *  by Jos Humberto Bento (1994), based on a DLL by Ron Burk
 */

#include <stdlib.h>
#include "printall.h"
#include <windows.h> 

#define WM_HELP_INQUIRE    (1)  /* asks what notifications you want */
#define WM_HELP_FUNCPTRS  (10)  /* supplies pointers to functions   */
#define HN_FUNCPTRS   (0x0010)  /* bit to receive WM_HELP_FUNCPTRS  */

typedef LONG (CALLBACK *WINHELPPTR)(LPSTR,WORD,DWORD);
typedef struct  HelpInstance
    {
    struct HelpInstance   *Next;
    HTASK           TaskHandle;
    WINHELPPTR      WinHelp;
    }           HelpInstance;

char buffer[100];

typedef struct arr_topicos
{
   char topico[50];
   int  n;
} ARRT;

int Canceled=FALSE;

#define MAXTOPICS       1000
ARRT  arr[MAXTOPICS];
int   nTopicos=0;

HelpInstance *HelpList;

HelpInstance    *NewHelpInstance(FARPROC FAR *FuncPtrs)
    {
    HelpInstance    *This;
    This        = (HelpInstance *)malloc(sizeof(HelpInstance));
    if(This != NULL)
        {
        This->Next          = NULL;
        This->TaskHandle    = GetCurrentTask();
        This->WinHelp       = (WINHELPPTR)FuncPtrs[16];
        if(HelpList)
            {
            This->Next  = HelpList;
            HelpList    = This;
            }
        else    /* else first on list */
            HelpList    = This;
        }
    return This;
    }

HelpInstance    *CurrentHelpInstance(void)
    {
    HTASK   ThisTask        = GetCurrentTask();
    HelpInstance    *Rover  = HelpList;

    while(Rover)    /* search linked list for matching task */
        if(Rover->TaskHandle == ThisTask)
            return Rover;
        else
            Rover   = Rover->Next;
    return Rover;
    }


HINSTANCE   ModuleHandle;

BOOL FAR PASCAL _export StopDlgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
   HWND hChild;

   switch (msg)
   {
   case WM_COMMAND:
      if(wParam==ID_CANCEL)
         {
         Canceled=TRUE;
         }        
      break;
   case WM_DESTROY:
       break;

   default:
       break;
    }
    return 0;

}

/* LibMain - store module handle, register clipboard format */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK LibMain(HINSTANCE ModuleHandle_,
    WORD DataSegment, WORD HeapSize, LPSTR CommandLine)
    {
    if(HeapSize > 0)
        UnlockData(0);
    ModuleHandle    = ModuleHandle_;
    return TRUE;
    }

/* WEP - return success */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK WEP(int ExitCode)
    {
    return 1;
    }


/* HelpYield() - Synchronize winhelp.exe macro execution */
static  void    HelpYield(void)
    {
    MSG     Message;

    while(PeekMessage(&Message, (HWND)NULL, 0, 0, PM_REMOVE))
        {
        if(Message.message == WM_QUIT)
            {
            PostQuitMessage(0);
            return;
            }
        else
            {
            TranslateMessage(&Message);
            DispatchMessage(&Message);
            }
        }
    }

/* PrintThis() - prints current topic in given help file */
void CALLBACK _export PAZ(char FAR* Topico, int n)
{
   if(nTopicos>MAXTOPICS)
      {  
      char b[150], c[150];
      
      LoadString(ModuleHandle, IDS_MAXTOPICS, b, sizeof(b) );
      LoadString(ModuleHandle, IDS_ERRORTITLE, c, sizeof(c) );
      MessageBox(GetActiveWindow(), b, c, 0 );
      return;
      }
   lstrcpy( arr[nTopicos].topico, (LPCSTR) Topico );
   arr[nTopicos++].n = n;
}

void CALLBACK _export PrintAll(char FAR *HelpFile)
{
   HelpInstance    *ThisHelp = CurrentHelpInstance();
   int i,j;
   HWND hWnd;
   DLGPROC stpdlg;

   stpdlg = (DLGPROC) MakeProcInstance( StopDlgProc, ModuleHandle );
   hWnd = CreateDialog( ModuleHandle, "STOPDLG", GetActiveWindow(), stpdlg );
   ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
       (DWORD)(void FAR *)"SaveMark(`PrintAllMark')");
   HelpYield();
   for(j=0;j<nTopicos;j++)
      {
      wsprintf(buffer, "JI(qchPath, \"%s\")", arr[j].topico );
      ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
          (DWORD)(void FAR *)buffer);
      HelpYield();
      for(i=0;i<arr[j].n;i++)
      {
          if(Canceled)
            break;
          ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
                  (DWORD)(void FAR *)"Print()");
          HelpYield();
          if(Canceled)
            break;
          ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
                  (DWORD)(void FAR *)"Next()");
          HelpYield();
          if(Canceled)
            break;
      }
   }
   Canceled=FALSE;
   DestroyWindow(hWnd);
   ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
       (DWORD)(void FAR *)"GoToMark(`PrintAllMark')");
   HelpYield();
   ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
       (DWORD)(void FAR *)"DeleteMark(`PrintAllMark')");
   HelpYield();
}

void CALLBACK _export PS(char FAR *HelpFile, char FAR* Topico, int n)
{
   HelpInstance    *ThisHelp = CurrentHelpInstance();
   int i,j;
   HWND hWnd;
   DLGPROC stpdlg;

   stpdlg = (DLGPROC) MakeProcInstance( StopDlgProc, ModuleHandle );
   hWnd = CreateDialog( ModuleHandle, "STOPDLG", GetActiveWindow(), stpdlg );
   ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
       (DWORD)(void FAR *)"SaveMark(`PrintSeqMark')");
   HelpYield();
   wsprintf(buffer, "JI(qchPath, \"%s\")", Topico );
   ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
        (DWORD)(void FAR *)buffer);
   HelpYield();
   for(i=0;i<n;i++)
   {
       if(Canceled)
         break;
       ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
               (DWORD)(void FAR *)"Print()");
       HelpYield();
       if(Canceled)
         break;
       ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
               (DWORD)(void FAR *)"Next()");
       HelpYield();
       if(Canceled)
         break;
   }
   Canceled=FALSE;
   DestroyWindow(hWnd);
   ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
       (DWORD)(void FAR *)"GoToMark(`PrintSeqMark')");
   HelpYield();
   ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
       (DWORD)(void FAR *)"DeleteMark(`PrintSeqMark')");
   HelpYield();
}

/* LDLLHandler - Special function that WinHelp calls implicitly */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
LONG CALLBACK _export LDLLHandler(WORD Message, LONG Param1, LONG Param2)
    {
    switch(Message)
        {
        case WM_HELP_INQUIRE   :
            return HN_FUNCPTRS;
        case WM_HELP_FUNCPTRS  :
            NewHelpInstance((FARPROC FAR *)Param1);
            return TRUE;
        }
    return FALSE;
    }

