#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <direct.h>
#include <dos.h>
#include "commdlg.h"
#include "drivinit.h"
#include "helputil.h"

static HelpInstance		HI;
static char				buffer[200];
static HDC				pDc=NULL;
static PRINTDLG			pd;      /* Common print dialog structure */
static BOOL				fAbort;		/* TRUE if the user has aborted the print job	 */
static HWND				hwndPDlg;		/* Handle to the cancel print dialog		 */
static PSTR				szTitle;		/* Global pointer to job title			 */
static char				sz[300];

HFS OpenHelpFileSystem()
{
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);
	return (HFS)ThisHelp->LGetInfo( GI_HFS, NULL );
}

HF OpenHelpFile( HFS hFs, LPCSTR File )
{
	HF					fh;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);
	lstrcpy( (LPSTR)buffer, File );
    fh = ThisHelp->HfOpenHfs ((HFS)ThisHelp->LGetInfo( GI_HFS, NULL ), buffer, fFSOpenReadOnly );
    if (!fh)
    	{
	    _strupr( buffer );
	    fh = ThisHelp->HfOpenHfs ((HFS)ThisHelp->LGetInfo( GI_HFS, NULL ), buffer, fFSOpenReadOnly );
	    if (!fh)
	    	{
		    _strlwr( buffer );
		    fh = ThisHelp->HfOpenHfs ((HFS)ThisHelp->LGetInfo( GI_HFS, NULL ), buffer, fFSOpenReadOnly );
	    	}
    	}
	return fh;
}

void GenExtractFile( LPSTR File, LPSTR FilePath )
{
	HF			fh;
	HFILE		hf;
	OFSTRUCT	of;
	UINT		nBytes;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);
	fh = OpenHelpFile( OpenHelpFileSystem(), File );
	if ( fh == NULL )
		{
		return;
		}
	GetTempFileName( 0, "AAA", 0, buffer );
	hf = OpenFile( buffer, &of, OF_CREATE | OF_READWRITE );
	if ( hf == HFILE_ERROR )
		{
		return;
		}
	for(;;)
		{
		nBytes = (UINT)ThisHelp->LcbReadHf( fh, file_buffer, sizeof(file_buffer) );
		_lwrite( hf, file_buffer, nBytes );
		if ( nBytes < sizeof(file_buffer) )
			break;
		}
	_lclose( hf );
	ThisHelp->RcCloseHf( fh );
	strcpy( FilePath, buffer );
}

void KillFile( LPSTR FilePath )
{
	if( !*FilePath )
		return;
	remove( FilePath );
}

int ReadChar( HFILE fh )
{
	char	ch;
	
	if ( _lread( fh, &ch, 1 ) < 1 )
		return EOF;
	return ch;
}

void ResetPrinter()
{
	DeleteDC( pDc );
	pDc = NULL;
}

BOOL CanceledPrinter()
{
	return pDc == NULL;
}

/****************************************************************************
 *									    *
 *  FUNCTION   : GetPrinterDC ()					    *
 *									    *
 *  PURPOSE    : Creates a printer display context for the printer  *
 *									    *
 *  RETURNS    : HDC   - A handle to printer DC.			    *
 *									    *
 ****************************************************************************/
HDC FAR PASCAL GetPrinterDC(void)
{

    HDC         hDC;
    LPDEVMODE   lpDevMode = NULL;
    LPDEVNAMES  lpDevNames;
    LPSTR       lpszDriverName;
    LPSTR       lpszDeviceName;
    LPSTR       lpszPortName;

	if(pDc!=NULL)
		return pDc;
	memset( &pd, 0, sizeof( PRINTDLG ));
	pd.lStructSize = sizeof( PRINTDLG );
	pd.hwndOwner = GetActiveWindow();
	pd.Flags = PD_RETURNDC;
    if (!PrintDlg((LPPRINTDLG)&pd))
        return(NULL);

    if (pd.hDC)
      {
        hDC = pd.hDC;
      }
    else
      {

        if (!pd.hDevNames)
            return(NULL);

        lpDevNames = (LPDEVNAMES)GlobalLock(pd.hDevNames);
        lpszDriverName = (LPSTR)lpDevNames + lpDevNames->wDriverOffset;
        lpszDeviceName = (LPSTR)lpDevNames + lpDevNames->wDeviceOffset;
        lpszPortName   = (LPSTR)lpDevNames + lpDevNames->wOutputOffset;
        GlobalUnlock(pd.hDevNames);

        if (pd.hDevMode)
            lpDevMode = (LPDEVMODE)GlobalLock(pd.hDevMode);

        hDC = CreateDC(lpszDriverName, lpszDeviceName, lpszPortName, (LPSTR)lpDevMode);

        if (pd.hDevMode && lpDevMode)
            GlobalUnlock(pd.hDevMode);
      }

    if (pd.hDevNames)
        GlobalFree(pd.hDevNames);
    if (pd.hDevMode)
        GlobalFree(pd.hDevMode);
	pDc = hDC;
    return(hDC);
}

/****************************************************************************
 *									    *
 *  FUNCTION   : AbortProc()						    *
 *									    *
 *  PURPOSE    : To be called by GDI print code to check for user abort.    *
 *									    *
 ****************************************************************************/
int FAR PASCAL __export AbortProc ( HDC hdc, WORD reserved )
{
    MSG msg;

    /* Allow other apps to run, or get abort messages */
    while (!fAbort && PeekMessage (&msg, NULL, NULL, NULL, TRUE))
		if (!hwndPDlg || !IsDialogMessage (hwndPDlg, &msg))
			{
	    	TranslateMessage (&msg);
	    	DispatchMessage  (&msg);
			}
    return !fAbort;
}

/****************************************************************************
 *									    *
 *  FUNCTION   : PrintDlgProc ()					    *
 *									    *
 *  PURPOSE    : Dialog function for the print cancel dialog box.	    *
 *									    *
 *  RETURNS    : TRUE  - OK to abort/ not OK to abort			    *
 *		 FALSE - otherwise.					    *
 *									    *
 ****************************************************************************/
BOOL FAR PASCAL __export PrintDlgProc(HWND hwnd, WORD msg, WORD wParam, LONG lParam)
{
	switch (msg)
		{
		case WM_COMMAND:
			/* abort printing if the only button gets hit */
			fAbort = TRUE;
			break;
		default:
			return FALSE;
		}
	return TRUE;
}

/****************************************************************************
 *									    									*
 *  FUNCTION   : locContarLinhas (LPSTR s)						    		*
 *									    									*
 *  PURPOSE    : Conta o nmero de linhas no texto		.		    		*
 *									    									*
 ****************************************************************************/

static int locContarLinhas( LPSTR s )
{
	int		nLines=0;
	
	for( ;*s;s++ )
		if ( *s=='\n' )
			nLines ++;
	if( (!*s) && *( s-1 ) != '\n' )
		nLines ++;
	return nLines;
}

/****************************************************************************
 *									    									*
 *  FUNCTION   : locLineLength (LPSTR s)						    		*
 *									    									*
 *  PURPOSE    : Conta o nmero de caracteres na linha	.		    		*
 *									    									*
 ****************************************************************************/

static int locLineLength( LPSTR s )
{
	int		nChars=0;
	
	for( ;*s;s++, nChars++ )
		if ( *s=='\n' || *s == '\r' )
			break;
	return nChars;
}

/****************************************************************************
 *									    									*
 *  FUNCTION   : locSkipLine (LPSTR s)							    		*
 *									    									*
 *  PURPOSE    : Conta o nmero de caracteres na linha	.		    		*
 *									    									*
 ****************************************************************************/

static int locSkipLine( LPSTR s )
{
	int		nChars=0;
	
	for( ;*s;s++, nChars++ )
		if ( *s=='\n' || *s == '\r' )
			break;
	if( *s== '\n' || *s == '\r' )
		{
		nChars++;
		s++;
		}
	if( *s== '\n' || *s == '\r' )
		{
		nChars++;
		s++;
		}
	return nChars;
}

/****************************************************************************
 *									    									*
 *  FUNCTION   : PrintFileFromHandle (HGLOBAL hGlob)	   					*
 *									    									*
 *  PURPOSE    : Prints the contents of the Text In The Globl Handle 		*
 *									    									*
 ****************************************************************************/

VOID FAR PASCAL PrintFileFromHandle(HGLOBAL hGlob, LPSTR Titulo)
{
    HDC     hdc;
    int     yExtPage;
    WORD    cch;
    LPSTR   pch;
    WORD    iLine;
    WORD    nLinesEc;
    FARPROC lpfnAbort;
    FARPROC lpfnPDlg;
    HWND    hwndPDlg;
    WORD    dy;
    int     yExtSoFar;
    WORD    fError = TRUE;
    LPSTR	Texto;
    long	curPos;
    HWND	hWndFrame = GetActiveWindow();

    /* Create the job title by loading the title string from STRINGTABLE */
    szTitle = sz;
    strcpy(sz, Titulo);
	cch = lstrlen( sz );
	
    /* Make instances of the Abort proc. and the Print dialog function */
    lpfnAbort = MakeProcInstance (AbortProc, hModule);
    if (!lpfnAbort)
		return;
    lpfnPDlg = MakeProcInstance (PrintDlgProc, hModule);
    if (!lpfnPDlg)
		{
	    FreeProcInstance (lpfnAbort);
	    return;
		};

    /* Initialize the printer */
    hdc = GetPrinterDC();
    if (!hdc)
		{
	    FreeProcInstance (lpfnAbort);
	    FreeProcInstance (lpfnPDlg);
	    return;
		};

    /* Disable the main application window and create the Cancel dialog */
    EnableWindow (hWndFrame, FALSE);
    hwndPDlg = CreateDialog (hModule, "PRINT", hWndFrame, lpfnPDlg);
    if (!hwndPDlg)
		{
	    FreeProcInstance (lpfnAbort);
	    FreeProcInstance (lpfnPDlg);
	    EnableWindow (hWndFrame, TRUE);
	    return;
		};
    ShowWindow (hwndPDlg, SW_SHOW);
    UpdateWindow (hwndPDlg);

    /* Allow the app. to inform GDI of the escape function to call */
    if (Escape (hdc, SETABORTPROC, 0, (LPSTR)lpfnAbort, NULL) < 0)
		{
	    FreeProcInstance (lpfnAbort);
	    FreeProcInstance (lpfnPDlg);
	    EnableWindow (hWndFrame, TRUE);
	    return;
		};

    /* Initialize the document */
    if (Escape (hdc, STARTDOC, cch, (LPSTR)sz, NULL) < 0)
		{
	    FreeProcInstance (lpfnAbort);
	    FreeProcInstance (lpfnPDlg);
	    EnableWindow (hWndFrame, TRUE);
	    return;
		};

    /* Get the height of one line and the height of a page */
    dy = HIWORD (GetTextExtent (hdc, "CC", 2));
    yExtPage = GetDeviceCaps (hdc, VERTRES);

    iLine     = 0;
    yExtSoFar = 0;
	Texto = (LPSTR) GlobalLock( hGlob );
    nLinesEc  = (WORD)locContarLinhas( Texto );
    curPos = 0;

    /* While more lines print out the text */
    while (iLine < nLinesEc)
    	{
		if (yExtSoFar + (int)dy > yExtPage)
			{
	    	/* Reached the end of a page. Tell the device driver to eject a
	     	* page
		    */
	    	if (Escape (hdc, NEWFRAME, 0, NULL, NULL) < 0 || fAbort)
				goto getout2;
		    yExtSoFar = 0;
			}

		/* Get the length and position of the line in the buffer
	 	 * and lock from that offset into the buffer */
		pch = Texto + curPos;
		cch = locLineLength( pch );

		/* Print the line and unlock the text handle */
		TextOut (hdc, 30, yExtSoFar, pch, cch);

		/* Test and see if the Abort flag has been set. If yes, exit. */
		if (fAbort)
	    	goto getout2;

		/* Move down the page */
		yExtSoFar += dy;
		curPos += locSkipLine( pch );
		iLine++;
	    }

    /* Eject the last page. */
    if (Escape (hdc, NEWFRAME, 0, NULL, NULL) < 0)
		goto getout2;

    /* Complete the document. */
    if (Escape (hdc, ENDDOC, 0, NULL, NULL) < 0)
    	{
getout2:
		/* Ran into a problem before NEWFRAME? Abort the document */
		Escape( hdc, ABORTDOC, 0, NULL, NULL);
    	}
    else
		fError=FALSE;

    /* Close the cancel dialog and re-enable main app. window */
    EnableWindow (hWndFrame, TRUE);
    DestroyWindow (hwndPDlg);

    /* Get rid of dialog procedure instances */
    FreeProcInstance (lpfnPDlg);

    FreeProcInstance (lpfnAbort);

	BringWindowToTop( hWndFrame );
    return;
}

#define		MINBUF			50000

HGLOBAL ReadFile( LPSTR File )
{
	HGLOBAL			hGlob = GlobalAlloc( GHND, MINBUF+1 );
	LPSTR			Texto;
	int				nTimes=0;
    HFILE			hf;
    OFSTRUCT		of;
	HelpInstance	*ThisHelp = &HI;
	HFS				hFs;
	HF				iHf;
	BOOL			Internal = FALSE;
	
	CurrentHelpInstance(ThisHelp);
	hf = OpenFile( File, &of, OF_READ );
	if( hf == HFILE_ERROR )
		{
		hFs = OpenHelpFileSystem();
		if ( hFs == NULL )
			return NULL;
		iHf = OpenHelpFile( hFs, File );
		Internal = TRUE;
		}
	for( ;; )
		{
		long	nBytes;
		
		Texto = GlobalLock( hGlob );
		if (!Internal)
			nBytes = _lread( hf, Texto + nTimes*MINBUF, MINBUF );
		else
			nBytes = ThisHelp->LcbReadHf( iHf, Texto + nTimes*MINBUF, MINBUF );
		*(Texto + nTimes*MINBUF + nBytes ) = '\0';
		GlobalUnlock( hGlob );
		if ( nBytes < MINBUF )
			break;
		nTimes ++;
		hGlob = GlobalReAlloc( hGlob, (nTimes+1) * MINBUF+1, GMEM_MOVEABLE );
		}
	if( !Internal )
		_lclose( hf );
	else
		ThisHelp->RcCloseHf( iHf );
	return hGlob;
}

long FileLength( LPSTR File )
{
    HFILE			hf;
    OFSTRUCT		of;
	HelpInstance	*ThisHelp = &HI;
	HFS				hFs;
	HF				iHf;
	BOOL			Internal = FALSE;
	long			nBytes;
	
	CurrentHelpInstance(ThisHelp);
	hf = OpenFile( File, &of, OF_READ );
	if( hf == HFILE_ERROR )
		{
		hFs = OpenHelpFileSystem();
		if ( hFs == NULL )
			return NULL;
		iHf = OpenHelpFile( hFs, File );
		Internal = TRUE;
		}
	if( Internal )
		nBytes = ThisHelp->LSeekHf( iHf, 0l, wFSSeekEnd );
	else
		nBytes = _llseek( hf, 0l, 2 );
	if( !Internal )
		_lclose( hf );
	else
		ThisHelp->RcCloseHf( iHf );
	return nBytes;
}

void CriarDirectorio( LPSTR Dir )
{
	char		b[9];
	unsigned	nd;
	
	strcpy(buffer, Dir );
	if (*(buffer+strlen(buffer)-1) =='\\')
		*(buffer+strlen(buffer)-1)='\0';
	_getcwd( sz, sizeof(sz) );
	if( !_chdir( buffer ) )
		{
	    _chdir( sz );
		return;
		}
	if( *(Dir+1) == ':' )
		{
		char ch = *Dir;
		if ( ch>='a' && ch <='z' )
			ch = ch - 'a' + 'A';
		if ( ch>='A' && ch <='Z' )
			_dos_setdrive( ch-'A'+1, &nd );
		Dir += 2;
		}
	if(*Dir=='\\')
		_chdir( "\\" );
    for(;;)
    	{
    	char *s = b;
    	
		for(;*Dir=='\\';Dir++);
    	for( ;*Dir && *Dir != '\\' && s-b<8; Dir++, s++ )
    		*s = *Dir;
    	*s = '\0';
    	if(*b)
    		{
    		_mkdir( b );
    		_chdir( b );
    		}
    	if((*Dir=='\\' && !*(Dir+1)) || !*Dir)
    		break;
    	}
	if( *(sz+1) == ':' )
		{
		char ch = *sz;
		if ( ch>='a' && ch <='z' )
			ch = ch - 'a' + 'A';
		if ( ch>='A' && ch <='Z' )
			_dos_setdrive( ch-'A'+1, &nd );
		}
    _chdir( sz );
}