#include <windows.h>
#include <string.h>
#include "..\compiler\comdef.h"
#include "winsize.h"

static HelpInstance	HI;
static char buffer[200];
static FILEHEADER file_header;
static LETTER letters[NUMLETTERS];
static struct tagCompleteDef
	{
	TOPICNAME	Name;
	TOPIC		Topic;
	} CompleteDef;

HFS OpenHelpFileSystem()
{
    HFS					hFs;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);
    hFs = (HFS)ThisHelp->LGetInfo( GI_HFS, NULL );
    return hFs;
}

HF OpenHelpFile( HFS hFs, LPCSTR File )
{
	HF					fh;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);
	lstrcpy( (LPSTR)buffer, File );
    fh = ThisHelp->HfOpenHfs ((HFS)ThisHelp->LGetInfo( GI_HFS, NULL ), buffer, fFSOpenReadOnly );
    if (!fh)
    	{
	    _strupr( buffer );
	    fh = ThisHelp->HfOpenHfs ((HFS)ThisHelp->LGetInfo( GI_HFS, NULL ), buffer, fFSOpenReadOnly );
	    if (!fh)
	    	{
		    _strlwr( buffer );
		    fh = ThisHelp->HfOpenHfs ((HFS)ThisHelp->LGetInfo( GI_HFS, NULL ), buffer, fFSOpenReadOnly );
	    	}
    	}
	return fh;
}

int FindChar( char *s, char ch )
{
	int i;
	
	for(i=0; *s && *s!=ch; s++, i++ );
	if(!*s)
		return -1;
	return i;
}

BOOL ExistsFile( LPSTR s )
{
	OFSTRUCT	of;
	
	return OpenFile( s, &of, OF_EXIST ) != HFILE_ERROR;
}


BOOL ExistsInternalFile( LPSTR s )
{
	HFS					hFs;
	HF					hF;
	HelpInstance		*ThisHelp = &HI;
	
	CurrentHelpInstance(ThisHelp);
	hFs = OpenHelpFileSystem();
	if ( !hFs )
		return FALSE;
	hF = OpenHelpFile( hFs, (LPCSTR)s );
	if( hF )
		ThisHelp -> RcCloseHf( hF );
	if ( hF )
		return TRUE;
	else
		return FALSE;
}

BOOL IsCompiled( LPSTR s, short * Version, BOOL Internal )
{
	OFSTRUCT	of;
	HFILE		hf;
	HelpInstance		*ThisHelp = &HI;

	if(Internal)
		{
		HF hF;
		HFS hFs;
		
		CurrentHelpInstance(ThisHelp);
		hFs = OpenHelpFileSystem();
		if ( !hFs )
			return FALSE;
		hF = OpenHelpFile( hFs, (LPCSTR)s );
		if ( !hF )
			return FALSE;
		if( ThisHelp->LcbReadHf( hF, (LPBYTE)&file_header, sizeof( file_header ) )<sizeof(file_header) )
			{
			ThisHelp -> RcCloseHf( hF );
			return FALSE;
			}
		ThisHelp -> RcCloseHf( hF );
		}
	else
		{
		hf = OpenFile( s, &of, OF_READ );
		if( hf == HFILE_ERROR )
			return FALSE;
		if( _lread( hf, &file_header, sizeof( file_header ) )< sizeof(file_header) )
			{
			_lclose( hf );
			return FALSE;
			}
		_lclose( hf );
		}
	if( file_header.magic != MAGIC || file_header.headerSize != sizeof(file_header) )
		return FALSE;
	* Version = file_header.ver;
	return TRUE;
}

long FilePos(long p)
{
	long	BasePos;

	BasePos = NUMLETTERS;
	BasePos *= sizeof(LETTER);
	BasePos += sizeof(FILEHEADER);
	BasePos += p*sizeof(COMPLETE);
	return BasePos;
};

BOOL stReadCompiledLine( char * TopicName, HF hF, HFILE hFile, HelpInstance *ThisHelp, COMPLETE * complete )
{
	long		FirstLine;
	long		LastLine;
	long		CurrentLine;
	char		ch;
	long		fsize;
	long		nRecords;
	COMPLETE	First;
	COMPLETE	Last;
	COMPLETE	Current;
	int			BasePos;

	BasePos = NUMLETTERS;
	BasePos *= sizeof(LETTER);
	BasePos += sizeof(FILEHEADER);
	fsize = (hFile == HFILE_ERROR ) ? ThisHelp->LSeekHf( hF, 0l, wFSSeekEnd ) : _llseek( hFile, 0l, 2 );
	if (hFile == HFILE_ERROR )
		ThisHelp->LSeekHf( hF, 0l, wFSSeekSet );
	else
		_llseek( hFile, 0l, 0 );
	if( ( ( hFile == HFILE_ERROR ) ?  ThisHelp->LcbReadHf( hF, (LPBYTE)&file_header, sizeof( file_header ) ) :
			_lread( hFile, &file_header, sizeof( file_header )))<sizeof(file_header) )
		{
		return FALSE;
		}
	if( file_header.magic != MAGIC || file_header.headerSize != sizeof(file_header) )
		return FALSE;
	if( _lread( hFile, &letters, sizeof( letters ) )<sizeof(letters) )
		return FALSE;
	nRecords = ( fsize - BasePos ) / sizeof(COMPLETE);
	if ( nRecords==0 )
		return FALSE;
	ch = *TopicName;
	if ( ch >='a' && ch <= 'z' )
		ch = ch -'a' + 'A';
	if ( ch<='A' || ch >= 'Z' )
		{
		FirstLine = 0;
		LastLine = nRecords-1;
		}
	else
		{
		ch = ch - 'A'; // Offset
		FirstLine = letters[ch].FirstLine;
		LastLine = letters[ch].LastLine;
		}
		if ( hFile == HFILE_ERROR )
			{
 			ThisHelp->LSeekHf( hF, FilePos( FirstLine ), wFSSeekSet );
 			ThisHelp->LcbReadHf( hF, (LPBYTE)&First, sizeof( COMPLETE ) );
 			ThisHelp->LSeekHf( hF, FilePos( LastLine ), wFSSeekSet );
 			ThisHelp->LcbReadHf( hF, (LPBYTE)&Last, sizeof( COMPLETE ) );
			}
		else
			{		
			_llseek( hFile, FilePos( FirstLine ), 0 );
			_lread( hFile, &First, sizeof(COMPLETE) );
			_llseek( hFile, FilePos( LastLine ), 0 );
			_lread( hFile, &Last, sizeof(COMPLETE) );
			}
	for(CurrentLine = FirstLine;;)
		{
		int comp;
		long CL = CurrentLine;

		CurrentLine = ( FirstLine + LastLine ) / 2;
		if ( hFile == HFILE_ERROR )
			{
 			ThisHelp->LSeekHf( hF, FilePos( CurrentLine ), wFSSeekSet );
 			ThisHelp->LcbReadHf( hF, (LPBYTE)&Current, sizeof( COMPLETE ) );
			}
		else
			{
			_llseek( hFile, FilePos( CurrentLine ), 0 );
			_lread( hFile, &Current, sizeof(COMPLETE) );
			}
		if( CurrentLine == CL )
			{
			if( !lstrcmp( TopicName, First.Name.name ) )
				{
				(*complete) = First;
				return TRUE;
				}
			if( !lstrcmp( TopicName, Last.Name.name ) )
				{
				(*complete) = Last;
				return TRUE;
				}
			return FALSE;
			}
		comp = lstrcmp( TopicName, Current.Name.name );
		if ( comp==0 )
			{
			(*complete) = Current;
			return TRUE;
			}
		if( comp < 0 )
			{
			Last = Current;
			LastLine = CurrentLine;
			}
		else
			{
			First = Current;
			FirstLine = CurrentLine;
			}
		}
	return FALSE;
}
