/*
 *  printall.c - WinHelp DLL to print all topics from a help file.
 *  by Jos Humberto Bento (1994), based on a DLL by Ron Burk
 */

#include <stdlib.h>
#include "utils.h"
#include <windows.h>               
#include <shellapi.h>

#define WM_HELP_INQUIRE    (1)  /* asks what notifications you want */
#define WM_HELP_FUNCPTRS  (10)  /* supplies pointers to functions   */
#define HN_FUNCPTRS   (0x0010)  /* bit to receive WM_HELP_FUNCPTRS  */

typedef LONG (CALLBACK *WINHELPPTR)(LPSTR,WORD,DWORD);
typedef struct  HelpInstance
    {
    struct HelpInstance   *Next;
    HTASK           TaskHandle;
    WINHELPPTR      WinHelp;
    }           HelpInstance;

char buffer[100];

HelpInstance *HelpList;

HINSTANCE   ModuleHandle;

HelpInstance    *NewHelpInstance(FARPROC FAR *FuncPtrs)
    {
    HelpInstance    *This;
    This        = (HelpInstance *)malloc(sizeof(HelpInstance));
    if(This != NULL)
        {
        This->Next          = NULL;
        This->TaskHandle    = GetCurrentTask();
        This->WinHelp       = (WINHELPPTR)FuncPtrs[16];
        if(HelpList)
            {
            This->Next  = HelpList;
            HelpList    = This;
            }
        else    /* else first on list */
            HelpList    = This;
        }
    return This;
    }

HelpInstance    *CurrentHelpInstance(void)
    {
    HTASK   ThisTask        = GetCurrentTask();
    HelpInstance    *Rover  = HelpList;

    while(Rover)    /* search linked list for matching task */
        if(Rover->TaskHandle == ThisTask)
            return Rover;
        else
            Rover   = Rover->Next;
    return Rover;
    }

/* LibMain - store module handle, register clipboard format */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK LibMain(HINSTANCE ModuleHandle_,
    WORD DataSegment, WORD HeapSize, LPSTR CommandLine)
    {
    if(HeapSize > 0)
        UnlockData(0);
    ModuleHandle    = ModuleHandle_;
    return TRUE;
    }

/* WEP - return success */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK WEP(int ExitCode)
    {
    return 1;
    }


/* HelpYield() - Synchronize winhelp.exe macro execution */
static  void    HelpYield(void)
    {
    MSG     Message;

    while(PeekMessage(&Message, (HWND)NULL, 0, 0, PM_REMOVE))
        {
        if(Message.message == WM_QUIT)
            {
            PostQuitMessage(0);
            return;
            }
        else
            {
            TranslateMessage(&Message);
            DispatchMessage(&Message);
            }
        }
    }
  
/* ForceIcon() - Forces the specified icon into the window */
void CALLBACK _export ForceIcon(LPSTR iconFile, WORD  nIndex)
{
   HelpInstance    *ThisHelp = CurrentHelpInstance();
	HICON				hIcon;
	
	hIcon = ExtractIcon( ModuleHandle, iconFile, nIndex );
	if (hIcon!=(HICON)NULL && hIcon!=(HICON)1)
		SetClassWord( GetActiveWindow( ), GCW_HICON, (WORD)hIcon );
}
  
/* LDLLHandler - Special function that WinHelp calls implicitly */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
LONG CALLBACK _export LDLLHandler(WORD Message, LONG Param1, LONG Param2)
    {
    switch(Message)
        {
        case WM_HELP_INQUIRE   :
            return HN_FUNCPTRS;
        case WM_HELP_FUNCPTRS  :
            NewHelpInstance((FARPROC FAR *)Param1);
            return TRUE;
        }
    return FALSE;
    }

